module Strategize
  # Operations let you define a block of code that you would like to
  # execute, and by calling process with a subject you can tell the
  # operation what object you would like to execute the code against.
  #
  # Operations are similar to Rules, but the block of code that you
  # pass in the constuctor does not have to be a predicate function.
  class Operation
    # @attr_reader name a descriptive name for the operation
    attr_reader :name

    # @attr_reader function A Proc or Lambda describing the code to be executed
    attr_reader :function

    # Create a new instance of Operation
    #
    # @param name [Symbol] a descriptive name for the operation
    # @param function [Proc] the code to be executed
    # @return [Operation]
    def initialize(name, function)
      unless function.respond_to?(:call)
        raise InvalidFunctionError, 'Invalid function in Operation'
      end

      @name = name
      @function = function
    end

    # Execute the function against a specific object
    #
    # @param subject [Object] will become self when executing the function
    # @return [void]
    def process(subject)
      subject.instance_exec(&@function)
    end
  end
end
