module Strategize
  # The Rule class represents a predicate function that
  # can be run against a specific subject.
  #
  # Define a new rule
  # Rule.new :rule_name, -> { code to execute }
  #
  # Evaluate a rule
  # rule = Rule.new :rule_name, -> { true }
  # rule.evaluate(nil) #=> true
  class Rule
    attr_reader :name, :predicate

    # Create a new rule
    #
    # @param name [Symbol] a descriptive name for the rule
    # @param predicate [Proc] code block to be executed
    def initialize(name, predicate)
      unless predicate.respond_to?(:call)
        raise InvalidPredicateError, 'Invalid predicate function passed'
      end

      @name = name
      @predicate = predicate
    end

    # Execute the predicate against the subject
    #
    # @param subject [Object]
    # @return [Boolean]
    def evaluate(subject)
      subject.instance_exec(&@predicate)
    end
  end
end
